#background_image
import os, shutil, json

def add_background_img(dashboard_path, page_id, img_path, alpha = 100, scaling_method = "Fit"):

	'''Add a background image to a dashboard page

	:param str dashboard_path: The path where the dashboard files are stored. (This is the top level directory containing the .pbip file and Report and SemanticModel folders). 
	:param str page_id: The unique id for the page you want to add the chart to. If you used this package's functions it will be in the format page1, page2, page3, page4, etc. If you manually created the page it will be a randomly generated UUID. To find a page's page id, consult the report > definition> pages > page.json file and look in the page order list. 
	:param str img_path: The path to the image you want to add. (Can be a relative path because the image is copied to the report folder). Allowed image types are whatever PBI allows manually, so probably at least jpeg and png
	:param int alpha: The transparency of the background image. Must be a whole integer between 1 and 100. 
	:param str scaling_method: The method used to scale the image available options include ["Fit", ]



	'''

	if type(alpha) is not int:
		raise TypeError("alpha (the transparency value) must be an integer between 1-100")

	if (alpha > 100) or (alpha < 0):
		raise ValueError("alpha (the transparency value) must be an integer between 1-100")

	# file paths
	report_name = os.path.basename(dashboard_path)
	img_name = os.path.basename(img_path)

	report_folder = os.path.join(dashboard_path, f'{report_name}.Report')
	definitions_folder = os.path.join(report_folder, "definition")

	page_json_path = os.path.join(definitions_folder, f"pages/{page_id}/page.json")
	report_json_path = os.path.join(definitions_folder, "report.json")


	registered_resources_folder = os.path.join(report_folder, "StaticResources/RegisteredResources")

	# This is the location of the image within the dashboard
	registered_img_path = os.path.join(registered_resources_folder, img_name)


	# Upload image to dashboard's registered resources ---------------------------------------------------

	# create registered resources folder if it doesn't exist
	if not os.path.exists(registered_resources_folder):
		os.makedirs(registered_resources_folder)

	# move image to registered resources folder
	shutil.copy(img_path, registered_img_path)


	# add new registered resource (the image) to report.json ----------------------------------------------
	with open(report_json_path,'r') as file:
		report_json = json.load(file)


	# add the image as an item to the registered resources items list
	for dict in report_json["resourcePackages"]:
		if dict["name"] == "RegisteredResources":
			dict["items"].append(
				                  {
                                    "name": img_name,
                                    "path": img_name,
                                    "type": "Image"
                                   }   
        	                    )



	#print(report_json)

   
	# write to file
	with open(report_json_path,'w') as file:
		json.dump(report_json, file, indent = 2)




	# Add image to page -------------------------------------------------------------------------------
	with open(page_json_path,'r') as file:
		page_json = json.load(file)


	# add the image to the page's json
	page_json["objects"]["background"] = [
      {
        "properties": {
          "image": {
            "image": {
              "name": {
                "expr": {
                  "Literal": {
                    "Value": f"'{img_name}'"
                  }
                }
              },
              "url": {
                "expr": {
                  "ResourcePackageItem": {
                    "PackageName": "RegisteredResources",
                    "PackageType": 1,
                    "ItemName": img_name
                  }
                }
              },
              "scaling": {
                "expr": {
                  "Literal": {
                    "Value": f"'{scaling_method}'"
                  }
                }
              }
            }
          },
          "transparency": {
            "expr": {
              "Literal": {
                "Value": f"{alpha}D"
              }
            }
          }
        }
      }
    ]

	

	

   
	# write to file
	with open(page_json_path,'w') as file:
		json.dump(page_json, file, indent = 2)








import os, re, shutil

from pathlib import Path
from importlib import resources


import PBI_dashboard_creator.update_diagramLayout as PBI_DL
import PBI_dashboard_creator.update_model_file as PBI_model              # internal function to add data to model.tmdl



def add_tmdl_dataset(dashboard_path, data_path = None, add_default_datetable = True):
	
	'''Add a locally stored TMDL file to the dashboard

	:param str dashboard_path: The path where the dashboard files are stored. (This is the top level directory containing the .pbip file and Report and SemanticModel folders). 
	:param str data_path: The path where the tmdl file is stored.
	:param boolean add_default_datetable: Do you want the TMDL file you add to be our team's custom date table? This will allow you to create your own date heirarchies instead of using time intelligence

	TMDL is a data storage format automatically created by power BI consisting of a table and column definitions and the M code used to generate the dataset. 
	In practice this means that you can copy datasets between dashboards. You can use this function to automatically copy the TMDL files at scale
	Potential pitfalls: M needs full paths to load data. If the new dashboard doesn't have access to the same data as the old dashboard, the data copying may fail. 



	'''

	if data_path is not None and add_default_datetable is True:
		raise ValueError("If you are providing a path to a tmdl dataset, the add_default_datetable argument can't be set to True")	
	




	# file paths
	report_name = os.path.basename(dashboard_path)

	semantic_model_folder = os.path.join(dashboard_path, f'{report_name}.SemanticModel' )
	definitions_folder = os.path.join(semantic_model_folder, "definition")
	tables_folder = os.path.join(definitions_folder, 'tables')


	# create tables folder if it doesn't already exist
	if not os.path.exists(tables_folder):
		os.makedirs(tables_folder)


	# check to see if we're importing the default DateTable.tmdl file
	# if the add default datetable argument is true
	# we'll use the path to the default datetable instead of the user supplied data_path
	# for the path to the datable we're going to add
	if add_default_datetable and data_path is None:

		# define the path we'll move the table to
		tmdl_dataset_path = os.path.join(tables_folder, "DateTable.tmdl")
		dataset_name = "DataTable"
		

		data_path = str(resources.files("PBI_dashboard_creator.dashboard_resources.python_resources").joinpath("DateTable.tmdl"))


		# copy date table from package resources to table folder
		shutil.copy(data_path, tmdl_dataset_path)

		# change the data_path to be the newly copied file
		data_path = tmdl_dataset_path

	else:
		# define the path we'll move the table to
		tmdl_dataset_path = os.path.join(tables_folder, os.path.basename(data_path))

		# dateset_name
		# extract the dataset name from the tmdl file's path
		# extract bits of names for later
		path_end = os.path.basename(data_path)
		split_end = os.path.splitext(path_end)

		dataset_name = split_end[0]

		# otherwise we move the tmdl file defined by the user to the tables folder
		# add the new tmdl file to the tables folder
		shutil.move(data_path, tmdl_dataset_path)





	# dateset_name -----------------------------------------------------------------------------------------------
	# read the whole table.tmdl file in andmake it a giant blob for regex
	file_content = ""

	with open(data_path) as file: 

		# list comprehension
		# all lines have all the white spaces and \n and \t striped 
		# They're then joined together using the .join function and ~ as a seperator
		file_content = "~".join(re.sub('\t?', '', line).rstrip() for line in file)


	# pull out just the dataset_id using regex
	m = re.search("(?<=lineageTag: ).*?(?=~~column)", file_content )


	dataset_id = m.group(0)




	# update the diagramLayout file to include the new date table\
	PBI_DL.update_diagramLayout(dashboard_path = dashboard_path, dataset_name = dataset_name, dataset_id = dataset_id)

	# update the model.tmdl file to include the new datetable
	PBI_model.update_model_file(dashboard_path = dashboard_path, dataset_name = dataset_name)






import  os, json, re

def add_text_box(text, dashboard_path, page_id, text_box_id, height, width,
 x_position, y_position, z_position = 6000, tab_order=-1001, 
  font_weight = "bold", font_size=32, font_color="#000000"):
    
    '''Add a text box to a page

    :param str text: The text you want to display in the box
    :param str dashboard_path: The path where the dashboard files are stored. (This is the top level directory containing the .pbip file and Report and SemanticModel folders). 
    :param str page_id: The unique id for the page you want to add the background image to. If you used this package's functions it will be in the format page1, page2, page3, page4, etc. If you manually created the page it will be a randomly generated UUID. To find a page's page id, consult the report > definition> pages > page.json file and look in the page order list. 
    :param str text_box_id: Please choose a unique id to use to identify the text box. PBI defaults to using a UUID, but it'd probably be easier if you choose your own id.

    :param int height: Height of text box on the page
    :param int width: Width of text box on the page
 
    :param int x_position: The x coordinate of where you want to put the text box on the page. Origin is page's top left corner. 
    :param int y_position: The y coordinate of where you want to put the text box on the page. Origin is page's top left corner.
    :param int z_position: The z index for the visual. (Larger number means more to the front, smaller number means more to the back). Defaults to 6000
    :param int tab_order: The order which the screen reader reads different elements on the page. Defaults to -1001 for now. (I need to do more to figure out what the numbers correpond to. It should also be possible to create a function to automatically order this left to right top to bottom by looping through all the visuals on a page and comparing their x and y positions)
    
    
    :param str font_weight: This is an option to change the font's weight. Defaults to bold. Available options include: ["bold"]
    :param int font_size: The font size in pts. Must be a whole integer. Defaults to 32 pt
    :param int font_color: Hex code for the font color you'd like to use. Defaults to black (#000000)  

    This function creates a new text box on a page. 
    '''

    # checks --------------------------------------------------------------------------------------------------------------

    # hex code
    # source: https://gist.github.com/dmartin4820/a53e18871d9490277b26ce21fd191af5
    #hex_match = re.search("/^#?([0-9a-f]{6}|[0-9a-f]{3});$/", font_color)

    #if hex_match is None:
        #raise ValueError("The hex code you provided for the font_color appears to be invalid! Please double check it.")

    
    # file paths -------------------------------
    report_name = os.path.basename(dashboard_path)
    
    pages_folder = os.path.join(dashboard_path, f'{report_name}.Report/definition/pages')
    page_folder_path = os.path.join(pages_folder, page_id)
    
    visuals_folder = os.path.join(page_folder_path, "visuals")
    new_visual_folder = os.path.join(visuals_folder, text_box_id)
    visual_json_path = os.path.join(new_visual_folder, "visual.json")

    # checks ---------------------------------------------------------

	# page exists? 
    if os.path.isdir(page_folder_path) is not True:
        raise NameError(f"Couldn't find the page folder at {page_folder_path}")
    
    # chart id unique? 
    if os.path.isdir(new_visual_folder) is True:
        raise ValueError(f'A visual with that text_box_id already exists! Try using a different text_box_id')
    
    else: 
        os.makedirs(new_visual_folder)
        
        
    text_box_json = {
  "$schema": "https://developer.microsoft.com/json-schemas/fabric/item/report/definition/visualContainer/1.3.0/schema.json",
  "name": text_box_id,
  "position": {
    "x": x_position,
    "y": y_position,
    "z": z_position,
    "height": height,
    "width": width,
    "tabOrder": tab_order 
  },
  "visual": {
    "visualType": "textbox",
    "objects": {
      "general": [
        {
          "properties": {
            "paragraphs": [
              {
                "textRuns": [
                  {
                    "value": text,
                    "textStyle": {
                      "fontWeight": font_weight,
                      "fontSize": f"{font_size}pt",
                      "color": font_color
                    }
                  }
                ]
              }
            ]
          }
        }
      ]
    },
    "visualContainerObjects": {
      "background": [
        {
          "properties": {
            "show": {
              "expr": {
                "Literal": {
                  "Value": "false"
                }
              }
            }
          }
        }
      ]
    },
    "drillFilterOtherVisuals": True
  }
    }

    # Write out the new json 
    with open(visual_json_path, "w") as file:
        json.dump(text_box_json, file, indent = 2)




import os, json

import PBI_dashboard_creator.add_text_box as PBI_text_box


def add_new_page(dashboard_path, page_name, title = None, subtitle = None):

	'''Create a new blank dashboard page

	:param str dashboard_path: The path where the dashboard files are stored. (This is the top level directory containing the .pbip file and Report and SemanticModel folders). 
	:param str page_name: The display name for the page you just created. This is differnt from the page_id which is only used internally. 
	:param str title: Title to put at the top of the page. This under the hood calls the add_text_box() function. If you would like more control over the title's appearance use that function instead.
	:param str sub_title: Title to put at the top of the page. This under the hood calls the add_text_box() function. If you would like more control over the title's appearance use that function instead.


	:returns: new_page_id: The unique id for the page you just created. If you used this function it will be in the format page1, page2, page3, page4, etc. If you manually create a page it will be a randomly generated UUID. To find a page's page id, consult the report > definition> pages > page.json file and look in the page order list. 
  
	'''

	# file paths
	report_name = os.path.basename(dashboard_path)
	pages_folder = os.path.join(dashboard_path, f'{report_name}.Report/definition/pages' )
	pages_json_path = os.path.join(pages_folder, "pages.json")

	# determine number of pages
	with open(pages_json_path,'r') as file:
		pages_list = json.load(file)

		# determine number of pages
		n_pages = len(pages_list["pageOrder"])

		# create a new page id based on existing number of pages
		new_page_id = f"page{n_pages + 1}"

		# add the new page id to the pageOrder list
		pages_list["pageOrder"].append(new_page_id)
    
  
	# write to file
	with open(pages_json_path,'w') as file:
		json.dump(pages_list, file, indent = 2)

	
	# create a folder for the new page
	new_page_folder = os.path.join(pages_folder, new_page_id)
	new_page_json_path = os.path.join(new_page_folder, "page.json")
	os.makedirs(new_page_folder)

	# create a new json file for the new page
	new_page_json = {"$schema": "https://developer.microsoft.com/json-schemas/fabric/item/report/definition/page/1.2.0/schema.json",
	                  "name": new_page_id,
	                  "displayName": page_name,
	                  "displayOption": "FitToPage",
	                  "height": 720,
	                  "width": 1280,
					  "objects":{}}


	# write to file
	with open(new_page_json_path, "w") as file:
		json.dump(new_page_json, file, indent = 2)


	# Add title and subtitle if requested 
	if title is not None:
		PBI_text_box.add_text_box(text = title,
             dashboard_path= dashboard_path,
               page_id= new_page_id,
                 text_box_id= f"{new_page_id}_title", 
                 height=68,
                   width=545,
                     x_position = 394, 
                     y_position = 44)

	if subtitle is not None:
		PBI_text_box.add_text_box(text = subtitle,
             dashboard_path= dashboard_path,
               page_id= new_page_id,
                 text_box_id= f"{new_page_id}_subtitle", 
                 height=38,
                   width=545,
                     x_position = 450, 
                     y_position = 90,
                     font_size = 14)



	return new_page_id

